@@ -1,15 +1,75 @@
 import numpy as np
-import logging
 import pandas as pd
 import numba
 from numba import prange
-import jupyter
+import requests
+import os
+import json
+import time
 
 
-logging.basicConfig(format='%(asctime)s\t\t%(message)s', level=logging.ERROR)
+def get_pv_data(latitude, longitude, token, output_folder='None'):  # ToDo update to require  output folder
+    api_base = 'https://www.renewables.ninja/api/'
+    s = requests.session()
+    # Send token header with each request
+    s.headers = {'Authorization': 'Token ' + token}
+
+    if output_folder == 'None':
+        out_dir = os.getcwd() + '/../../pv_data'
+
+        try:
+            os.mkdir(out_dir)
+        except FileExistsError:
+            pass
+    else:
+        out_dir = output_folder
+
+    out_path = os.path.join(out_dir, 'pv_data_lat_{}_long_{}.csv'.format(latitude, longitude))
+
+    url = api_base + 'data/pv'
+
+    args = {
+        'lat': latitude,
+        'lon': longitude,
+        'date_from': '2020-01-01',
+        'date_to': '2020-12-31',
+        'dataset': 'merra2',
+        'capacity': 1.0,
+        'system_loss': 0.1,
+        'tracking': 0,
+        'tilt': 35,
+        'azim': 180,
+        'format': 'json',
+        'local_time': True,
+        'raw': True
+    }
+
+    try:
+        r = s.get(url, params=args)
+
+        # Parse JSON to get a pandas.DataFrame of data and dict of metadata
+        parsed_response = json.loads(r.text)
+
+    except json.decoder.JSONDecodeError:
+        print('API maximum hourly requests reached, waiting one hour', time.ctime())
+        time.sleep(3700)
+        print('Wait over, resuming API requests', time.ctime())
+        r = s.get(url, params=args)
+
+        # Parse JSON to get a pandas.DataFrame of data and dict of metadata
+        parsed_response = json.loads(r.text)
+
+    data = pd.read_json(json.dumps(parsed_response['data']), orient='index')
+
+    df_out = pd.DataFrame(columns=['time', 'ghi', 'temp'])
+    df_out['ghi'] = (data['irradiance_direct'] + data['irradiance_diffuse']) * 1000
+    df_out['temp'] = data['temperature']
+    df_out['time'] = data['local_time']
+
+    df_out.to_csv(out_path)
 
 
-def read_environmental_data(path, skiprows=24, skipcols=1):
+def read_environmental_data(path, skiprows=24, skipcols=1): # ToDo update skipcols or index in get_pv_data
     data = pd.read_csv(path, skiprows=skiprows)
     ghi_curve = data.iloc[:, 0 + skipcols].values
     temp = data.iloc[:, 1 + skipcols].values
@@ -32,7 +92,7 @@ def pv_generation(temp, ghi, pv_capacity, load, inv_eff):
     return net_load, pv_gen
 
 
-@numba.njit  # ToDo ensure works with battery_size = 0
+@numba.njit
 def diesel_dispatch(hour, net_load, diesel_capacity, fuel_result, annual_diesel_gen, soc, inv_eff, n_dis, n_chg,
                     battery_size,
                     chg_max=1,  # Max share of battery capacity that can be charged in one hour
@@ -132,7 +192,7 @@ def soc_and_battery_usage(net_load, diesel_gen, n_dis, inv_eff, battery_size, n_
 
 @numba.njit
 def unmet_demand_and_excess_gen(unmet_demand, soc, n_dis, battery_size, n_chg, excess_gen, dod,
-                                hour, battery_life, dod_max, battery_use, remaining_load):
+                                hour, dod_max, battery_use, remaining_load):
 
     if battery_size > 0:
         if soc < 0:
@@ -155,15 +215,12 @@ def unmet_demand_and_excess_gen(unmet_demand, soc, n_dis, battery_size, n_chg, e
     if 1 - soc > dod:
         dod = 1 - soc
 
-    if hour == 23:  # The battery wear during the last day is calculated
-        battery_life = battery_life + battery_use / (531.52764 * max(0.1, dod * dod_max) ** -1.12297)
-
-    return unmet_demand, soc, excess_gen, dod,  battery_life
+    return unmet_demand, soc, excess_gen, dod
 
 
 @numba.njit
 def hourly_optimization(battery_size, diesel_capacity, net_load, hour_numbers, inv_eff, n_dis, n_chg, dod_max,
-                        energy_per_hh):
+                        energy_per_hh, full_life_cycles):
     fuel_result = 0
     battery_life = 0
     soc = 0.5
@@ -191,11 +248,10 @@ def hourly_optimization(battery_size, diesel_capacity, net_load, hour_numbers, i
                                                                               battery_use, soc, hour, dod)
             annual_battery_use += hourly_battery_use
 
-        unmet_demand, soc, excess_gen, dod, battery_life = unmet_demand_and_excess_gen(unmet_demand, soc, n_dis,
+        unmet_demand, soc, excess_gen, dod = unmet_demand_and_excess_gen(unmet_demand, soc, n_dis,
                                                                                        battery_size, n_chg,
                                                                                        excess_gen, dod, hour,
-                                                                                       battery_life, dod_max,
-                                                                                       battery_use, load)
+                                                                                       dod_max, battery_use, load)
 
         soc_array.append(soc)
         diesel_gen_curve.append(diesel_gen)
@@ -203,18 +259,15 @@ def hourly_optimization(battery_size, diesel_capacity, net_load, hour_numbers, i
         battery_soc_curve.append(soc)
 
     if (battery_size > 0) & (annual_battery_use > 0):
-        battery_life_simple = round(3400 / annual_battery_use)  # Optimum case, 2500
+        battery_life_simple = round(full_life_cycles / annual_battery_use)  # Optimum case, 2500
         battery_life_simple = min(battery_life_simple, 20)
     else:
         battery_life_simple = 20
 
     condition = unmet_demand / energy_per_hh  # LPSP is calculated
     excess_gen = excess_gen / energy_per_hh
-    battery_life = round(1 / battery_life)
     diesel_share = annual_diesel_gen / energy_per_hh
 
-    battery_life = min(battery_life, 20)
-
     return diesel_share, battery_life_simple, condition, fuel_result, excess_gen, battery_soc_curve, diesel_gen_curve
 
 
@@ -223,14 +276,15 @@ def calculate_hybrid_lcoe(diesel_price, end_year, start_year, energy_per_hh,
                           fuel_usage, pv_panel_size, pv_cost, charge_controller, pv_om, diesel_capacity, diesel_cost,
                           diesel_om, inverter_life, load_curve, inverter_cost, diesel_life, pv_life, battery_life,
                           battery_size, battery_cost, dod_max, discount_rate):
+
     # Necessary information for calculation of LCOE is defined
     project_life = end_year - start_year
     generation = np.ones(project_life) * energy_per_hh
     generation[0] = 0
 
     # Calculate LCOE
-    sum_el_gen = 0  # np.zeros((len(battery_sizes), pv_no, diesel_no))
-    investment = 0  # np.zeros((len(battery_sizes), pv_no, diesel_no))
+    sum_el_gen = 0
+    investment = 0
     sum_costs = 0
     total_battery_investment = 0
     total_fuel_cost = 0
@@ -278,11 +332,11 @@ def calculate_hybrid_lcoe(diesel_price, end_year, start_year, energy_per_hh,
     return sum_costs / sum_el_gen, investment, total_battery_investment, total_fuel_cost, total_om_cost
 
 
-# @numba.njit
+@numba.njit
 def find_least_cost_option(configuration, temp, ghi, hour_numbers, load_curve, inv_eff, n_dis, n_chg, dod_max,
                            energy_per_hh, diesel_price, end_year, start_year, pv_cost, charge_controller, pv_om,
                            diesel_cost, diesel_om, inverter_life, inverter_cost, diesel_life, pv_life, battery_cost,
-                           discount_rate, lpsp_max, diesel_limit, simple=True, for_plots=False):
+                           discount_rate, lpsp_max, diesel_limit, full_life_cycles, simple=True, for_plots=False):
 
     pv = configuration[0]
     battery = configuration[1]
@@ -293,7 +347,7 @@ def find_least_cost_option(configuration, temp, ghi, hour_numbers, load_curve, i
     # For the number of diesel, pv and battery capacities the lpsp, battery lifetime, fuel usage and LPSP is calculated
     diesel_share, battery_life, lpsp, fuel_usage, excess_gen, battery_soc_curve, diesel_gen_curve = \
         hourly_optimization(battery, diesel, net_load, hour_numbers, inv_eff, n_dis, n_chg,
-                            dod_max, energy_per_hh)
+                            dod_max, energy_per_hh, full_life_cycles)
     if battery == 0:
         battery_life = 1
     else:
@@ -368,10 +422,6 @@ def pv_diesel_hybrid(
         start_year,
         end_year,
         diesel_price,
-        chg_max=1,  # Max share of battery capacity that can be charged in one hour
-        dschg_max=1,  # Max share of battery capacity that can be discharged in one hour
-        kibam_c=0,  #  c parameter of KiBaM model (if 0, KiBaM will not be used)
-        kibam_k=0,  # k parameter of KiBaM model (if 0, KiBaM will not be used)
         max_cycles=950,  # cycles to failure at dod_max
         dod_max=0.9,  # maximum depth of discharge of battery
         diesel_cost=565,  # diesel generator capital cost, USD/kW rated power  # ToDo check
